import os
import datetime

from apiflask import abort, APIBlueprint
from flask import jsonify

from db import db
from helper_functions import get_user_id_from_username, get_username_or_abort_401
from models import Transaction
from scheme import TransactionSchema
from auth import auth

transaction_blueprint = APIBlueprint('transaction', __name__, url_prefix='/api')
__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))


@transaction_blueprint.route('/transaction', methods=['POST'])
@transaction_blueprint.output((), 200)
@transaction_blueprint.input(schema=TransactionSchema)
@transaction_blueprint.auth_required(auth)
@transaction_blueprint.doc(summary="Adds new transaction", description="Adds new transaction for current user")
def add_transaction(data):
    username = get_username_or_abort_401()

    check_if_transaction_data_exists(data)

    new_transaction = Transaction(
        user_id=get_user_id_from_username(username),
        symbol=data['symbol'],
        time=datetime.datetime.strptime(data['time'], '%Y-%m-%dT%H:%M:%S.%fZ'),
        count=data['count'],
        price=data['price']
    )
    db.session.add(new_transaction)
    db.session.commit()

    return jsonify({"status": 200, "text": "Successfully added transaction", "data": new_transaction.as_dict()})


@transaction_blueprint.route('/transactions', methods=['GET'])
@transaction_blueprint.output(TransactionSchema(), 200)
@transaction_blueprint.auth_required(auth)
@transaction_blueprint.doc(summary="Returns all transactions", description="Returns all transactions for current user")
def get_transaction():
    username = get_username_or_abort_401()

    return_transactions = []
    transactions = db.session.query(Transaction).filter_by(user_id=get_user_id_from_username(username)).all()

    if transactions is not None:
        for row in transactions:
            return_transactions.append(row.as_dict())

    return jsonify({"status": 200, "text": "Successfully loaded transactions", "data": return_transactions})


def check_if_transaction_data_exists(data):
    if "symbol" not in data:
        abort(400, message="Symbol missing")

    if data['symbol'] == "" or data['symbol'] is None:
        abort(400, message="Symbol missing")

    if "time" not in data:
        abort(400, message="Time missing")

    if data['time'] == "" or data['time'] is None:
        abort(400, message="Time missing")

    if "count" not in data:
        abort(400, message="Count missing")

    if data['count'] == "" or data['count'] is None:
        abort(400, message="Count missing")

    if "price" not in data:
        abort(400, message="Price missing")

    if data['price'] == "" or data['price'] is None:
        abort(400, message="Price missing")
